# DEMO 使用教程

## step0: Avro 数据结构代码自动生成 (可选)

    请注意当前Demo中已生成了代码，无需执行此步骤，除非你非常熟悉Avro且对这里的schema做了修改。
    进入resources目录并执行：
    java -jar avro-tools-1.8.2.jar compile -string schema Record-ali.avsc Record.avsc ../java/

## step1: 改造 UserBusinessProcess 类，添加业务逻辑
    
    每一个Kafka消息经过反序列化并转换为 com.alibaba.dts.formats.avro.Record 对象后，会传递到
    UserBusinessProcess#onRecord 函数中进行下游的业务处理，Demo 中已有的实现仅提供一个示例：
    将 Record 对象以 JSON 字符串的格式打印出来或者解析为 SQL 语句打印出来。

    你可以在这个类中根据业务逻辑自由扩展自己的消费程序。如果之前已有阿里云 Avro 格式的数据订阅消费程序，
    可以将阿里云消费程序中对 Record 对象解析处理以及其后的代码逻辑迁移到 UserBusinessProcess 
    这个类中，由于这里的 Record 类可以认为是完全兼容阿里云 Avro 格式数据订阅消费程序中的对应的类
    （除个别非关键字段如 bornTimestamp，详见下面 Record 字段说明表格），因此代码的迁移比较容易，
    仅需要简单的适配和调试即可运行。

## step2: 代码编译打包

    进入 pom.xml 文件所在的目录并执行：
    mvn clean assembly:assembly -Dmaven.test.skip=true

## step3: 启动程序消费数据

    请注意本消费程序仅在腾讯云内网可消费数据，且消费端所在地域需要与订阅任务所在地域一致，否则 Kafka 
    地址无法连通，所以需要将 step2 中打包好的程序包上传到腾讯云服务器上执行：
    java -jar consumerDemo-avro-1.0-SNAPSHOT.jar --brokers xxx --topic xxx --group xxx --user xxx --password xxx --trans2sql true

    启动参数说明：
    | 参数      |       说明                      |
    | ----------| -------------------------------|
    | brokers   | Kafka 的内网访问地址（域名+端口号） |
    | topic     | Kafka topic 名称                |
    | group     | Kafka 消费组名称                 |
    | user      | Kafka 消费组账号名               |
    | password  | Kafka 消费组密码                 |
    | trans2sql | 是否将消费数据转换为 SQL 语句打印   |


## 附录1：Demo 文件与目录说明

| 文件/目录   |       说明                      |
| ----------| -------------------------------|
| resources/Record.avsc | 腾讯云avro数据结构定义文件（为了便于转换为阿里云的格式，这里借用了Record-ali.avsc中定义的基本类型如Integer） | 
| resources/Record-ali.avsc |  阿里云avro数据结构定义文件 |
| resources/Record-v6.avsc | 腾讯云老版本avro数据结构定义文件，请忽略 |
| resources/avro-tools-1.8.2.jar | avro代码生成工具，一般无需关注 |
| java/com/alibaba.dts.formats.avro | 阿里云avro数据结构生成代码目录 |
| java/com/tencent.subscribe.avro | 腾讯云avro数据结构生成代码目录 |
| java/com/ClassConvertor | 用于将腾讯云数据结构转阿里云数据结构的工具类，一般无需关注 |
| java/com/DataTypeParser | 基础数据结构解析工具类，如将时间类型解析为字符串，一般无需关注 |
| java/com/Deserializer | Avro数据反序列化工具类，将Kafka消息中的二进制数据反序列化为Avro定义的结构，一般无需关注 |
| java/com/SubscribeFromKafka | 从Kafka中消费数据并进行反序列化的类，Demo主函数和核心流程都在这里实现 |
| java/com/UserBusinessProcess | 用户业务逻辑类，Kafka消息反序列化并转为阿里云的数据格式后传递到这里执行一定的业务逻辑，本Demo中仅实现了打印或者解析为SQL语句的功能。请根据你的业务需要自行在此扩展，也可以将之前阿里云消费程序中的业务处理代码迁移到此，简单适配后即可编译运行 |


## 附录2：协议文件说明

### com.alibaba.dts.formats.avro.Record 字段说明

| 字段名称               |     类型       |含义                      |
| ---------------       | --------      | ----------               |
| version               |   int         | 协议版本，当前版本为2       |
| id                    |   long        | 全局递增Id                |
| sourceTimestamp       |   long        | 当前record在binlog event里的时间戳，秒级 |
| sourcePosition        |   String      | 当前record的在binlog中结束的偏移量，格式为 End_log_pos@binlog文件编号。比如，当前record位于文件mysql-bin.000004中，结束偏移量为2196，则其值为"2196@4" |
| safeSourcePosition    |   String      | 当前事务在binlog中开始的偏移量，格式同上   |
| sourceTxid            |   String      | 事务Id，只有COMMIT记录才会有 |
| source                |   com.alibaba.dts.formats.avro.Source  | 源库类型与版本信息 |
| operation             |   com.alibaba.dts.formats.avro.Operation | 消息类型，支持:INSERT,UPDATE,DELETE,DDL,BEGIN,COMMIT,ROLLBACK,HEARTBEAT,CHECKPOINT|
| objectName            |   String      | 库名.表名        |
| processTimestamps     |   List<Long>  | 数据处理流程中的时间戳，未实现   |
| tags                  |   Map<String,String>  | 一些额外信息，这里实现了thread_id,server_id,pk_uk_info,readerThroughoutTime,GTID,GTID_SET |
| fields                |   Object | 表中各列的定义，也就是表头          |
| beforeImages          |   Object | dml执行前该行的数据，如果是insert消息，该数组为null。数组中元素共有12种类型:Integer,Character,Decimal,Float,Timestamp,DateTime,TimestampWithTimeZone,BinaryGeometry,TextGeometry,BinaryObject,TextObject,EmptyObject，详见demo中定义              |
| afterImages           |   Object | dml执行后该行的数据，如果是delete消息，该数组为null。数组中元素共有12种类型:Integer,Character,Decimal,Float,Timestamp,DateTime,TimestampWithTimeZone,BinaryGeometry,TextGeometry,BinaryObject,TextObject,EmptyObject，详见demo中定义              |
| bornTimestamp         |   long   | 未实现，腾讯云中无对应字段  |


### mysql列属性

Field是描述列属性的对象。包含2个属性：

+ name 列名
+ dataTypeNumber 是binlog中记录的数据类型。取值详见：https://dev.mysql.com/doc/internals/en/com-query-response.html


### mysql数据类型转换逻辑

在avro协议中，将mysql类型转换成了以下11中格式。

#### Integer

对应数据库中的整数类型，如：TINYINT SMALLINT MEDIUMINT INT BIGINT 和 BIT。  
拥有两个属性：precision表示整数占有的字节数；value为整数值。对于BIT类型，precision固定为8。

#### Character

对应数据库中的字符类型，如：CHAR VARCHAR TEXT等。

#### Float

对应数据库中的浮点类型，如：FLOAT 和 DOUBLE。 precision和scale分别为information_schema.COLUMNS表中的数据。    
注意，如果表或者列是在订阅任务启动之后新增的，则无法获取到precision和scale。

#### Decimal

对应数据库中DECIMAL。

#### Timestamp

对应数据库中的TIMESTAMP。其中timestamp单位为秒，millis单位为毫秒，范围为[0,999]。

#### DateTime

对应数据库中的其他时间类型

#### TimestampWithTimeZone

暂未使用

#### BinaryGeometry

对应数据库中的geometry及其子类。

#### TextGeometry

暂未使用

#### BinaryObject

对应数据库中的二进制类型，如BINARY VARBINARY 和 BLOB等。

#### TextObject

对应数据库中的文本类型，如ENUM SET 和 JSON。

#### EmptyObject

对应NULL
如果源库的**binlog_row_image**变量设置为**MINIMAL**，binlog不会记录未修改列，此时输出null；
如果表中数据为null，此时输出EmptyObject，反序列化后是字符串null。
