import com.alibaba.dts.formats.avro.Field;
import com.alibaba.dts.formats.avro.Operation;
import com.alibaba.dts.formats.avro.Source;
import com.alibaba.dts.formats.avro.SourceType;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.BeanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.tencent.subscribe.avro.Record;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ClassConvertor {
    private static final Logger log = LoggerFactory.getLogger(ClassConvertor.class);
    public static final String TAG_THREAD_ID = "thread_id";
    public static final String TAG_SERVER_ID = "server_id";
    public static final String TAG_PK_INFO = "pk_uk_info";
    public static final String TAG_READER_TIME = "readerThroughoutTime";
    public static final String TAG_GTID = "GTID";
    public static final String TAG_GTID_SET = "GTID_SET";

    public static com.alibaba.dts.formats.avro.Record convert2AliAvroRecord(Record record) {
        com.alibaba.dts.formats.avro.Record aliRecord = new com.alibaba.dts.formats.avro.Record();
        BeanUtils.copyProperties(record, aliRecord);
        // 以下处理不兼容的字段，包括 operation、tag、field 等
        try {
            aliRecord.setOperation(Operation.valueOf(record.getOperation().toString()));
        } catch (Exception e) {
            e.printStackTrace();
            log.error("unsupported operation type in ali avro: {}", record.getOperation());
        }
        aliRecord.setSource(new Source(SourceType.MySQL, record.getSourceVersion()));
        Map<String, String> tags = new HashMap<>();
        tags.put(TAG_THREAD_ID, record.getThreadId());
        tags.put(TAG_SERVER_ID, record.getServerId());
        tags.put(TAG_GTID, record.getGtid());
        tags.put(TAG_GTID_SET, record.getSyncedGtid());
        tags.put(TAG_READER_TIME, "" + record.getReaderThroughoutTime());
        JSONObject pkObject = new JSONObject();
        JSONArray pkNames = new JSONArray(record.getPkNames());
        pkObject.put("PRIMARY", pkNames);
        tags.put(TAG_PK_INFO, pkObject.toString());
        aliRecord.setTags(tags);
        // ali Record 把 ddl 也存到 afterImages 字段
        if (aliRecord.getOperation() == Operation.DDL) {
            aliRecord.setAfterImages(record.getSql());
        }
        // Field 字段深度转换
        List<com.tencent.subscribe.avro.Field> fields = record.getFields();
        List<Field> aliFields = new ArrayList<>();
        if (fields != null && fields.size() > 0) {
            for (int i = 0; i < fields.size(); i++) {
                Field f = new Field();
                BeanUtils.copyProperties(fields.get(i), f);
                aliFields.add(f);
            }
        }
        aliRecord.setFields(aliFields);

        return aliRecord;
    }

}
