import com.tencent.subscribe.avro.Record;
import com.tencent.subscribe.avro.old.OldRecord;
import org.apache.avro.Schema;
import org.apache.avro.io.*;
import org.apache.avro.specific.SpecificDatumReader;
import org.apache.avro.specific.SpecificDatumWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.EOFException;

/**
 * @author dingweiran
 * @version 2022/6/1
 */
public class Deserializer {
    private static final Logger log = LoggerFactory.getLogger(Deserializer.class);
    private static final SpecificDatumReader<Record> reader = new SpecificDatumReader<>(Record.class);
    private static final SpecificDatumWriter<Record> writer = new SpecificDatumWriter<>(Record.class);


    public static byte[] avroSerialize(Record record) {
        try (ByteArrayOutputStream baos = new ByteArrayOutputStream()) {
            Encoder encoder = EncoderFactory.get().binaryEncoder(baos, null);
            writer.write(record, encoder);
            return baos.toByteArray();
        } catch (Throwable ex) {
            log.error("AvroSerializer: serialize record failed cause " + ex.getMessage(), ex);
            throw new RuntimeException(ex);
        }
    }

    public static Record avroDeserialize(byte[] data) {
        Decoder decoder = DecoderFactory.get().binaryDecoder(data, null);
        try {
            return reader.read(null, decoder);
        } catch (Exception ex) {
            if (ex instanceof EOFException) { // 新版本schema，旧版本数据
                return avroDeserializeOldData(data);
            }
            log.error("AvroDeserializer: deserialize record failed cause " + ex.getMessage(), ex);
            throw new RuntimeException(ex);
        }
    }

    public static Record avroDeserializeOldData(byte[] data) {
        Decoder decoder = DecoderFactory.get().binaryDecoder(data, null);
        try {
            Schema oldSchema = OldRecord.getClassSchema();
            Schema newSchema = Record.getClassSchema();
            SpecificDatumReader<Record> oldDataReader = new SpecificDatumReader<>(oldSchema, newSchema);
            return oldDataReader.read(null, decoder);
        } catch (Throwable ex) {
            log.error("AvroDeserializer: deserialize record failed cause " + ex.getMessage(), ex);
            throw new RuntimeException(ex);
        }
    }

}
